@@ -28,7 +28,10 @@
 #
 #**************************************************************************************
 #**************************************************************************************
+import os
 import sys
+from urllib.request import urlretrieve
+
 import rdflib
 from rdflib import Graph, URIRef, util
 import pandas as pd
@@ -37,10 +40,13 @@ from nidm.core import Constants
 import nidm.experiment.CDE
 import re
 import tempfile
-from os import path
+from os import path, environ
 import functools
 import hashlib
 import pickle
+import requests
+import json
+
 
 from joblib import Memory
 memory = Memory(tempfile.gettempdir(), verbose=0 )
@@ -64,6 +70,28 @@ def sparql_query_nidm(nidm_file_list,query, output_file=None, return_graph=False
     :return: dataframe | graph depending on return_graph parameter
     '''
 
+
+
+    if 'BLAZEGRAPH_URL' in environ.keys():
+        try:
+            # first make sure all files are loaded into blazegraph
+            for nidm_file in nidm_file_list:
+                OpenGraph(nidm_file)
+            logging.debug("Sending sparql to blazegraph: %s", query )
+            r2 = requests.post(url=environ['BLAZEGRAPH_URL'], params={'query': query}, headers={'Accept': 'application/sparql-results+json'})
+            content = json.loads( r2.content )
+            columns = {}
+            for key in content["head"]['vars']:
+                columns[key] = [x[key]['value'] for x in content['results']['bindings']]
+            df = pd.DataFrame(data=columns)
+            if (output_file is not None):
+                df.to_csv(output_file)
+            return df
+
+        except Exception as e:
+            print("Exception while communicating with blazegraph at {}: {}".format(environ['BLAZEGRAPH_URL'],e))
+
+
     #query result list
     results = []
 
@@ -143,9 +171,37 @@ def GetProjectsUUID(nidm_file_list,output_file=None):
 
         }
     '''
-    df = sparql_query_nidm(nidm_file_list,query, output_file=output_file)
+    df = sparql_query_nidm(nidm_file_list, query, output_file=output_file)
 
-    return df['uuid'].tolist()
+    return df['uuid'] if type(df['uuid']) == list else df['uuid'].tolist()
+
+def GetProjectLocation(nidm_file_list, project_uuid, output_file=None):
+    '''
+    This query will return the prov:Location value for project_uuid
+
+    :param nidm_file_list: List of one or more NIDM files to query across for list of Projects
+    :param output_file: Optional output file
+    :return: list of Project prov:Locations
+    '''
+
+    # SPARQL query to get project UUIDs
+    query = '''
+            PREFIX nidm:<http://purl.org/nidash/nidm#>
+            PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+            prefix prov: <http://www.w3.org/ns/prov#>
+
+            SELECT distinct ?location
+            Where {
+                {
+                    ?uuid rdf:type nidm:Project ;
+                        prov:Location ?location .
+                }
+
+            }
+        '''
+    df = sparql_query_nidm(nidm_file_list, query, output_file=output_file)
+
+    return df['location'].tolist()
 
 def testprojectmeta(nidm_file_list):
 
@@ -317,7 +373,6 @@ def GetParticipantIDs(nidm_file_list,output_file=None):
         PREFIX sio: <http://semanticscience.org/ontology/sio.owl#>
         PREFIX ndar: <https://ndar.nih.gov/api/datadictionary/v2/dataelement/>
         PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
-        PREFIX prov:<http://www.w3.org/ns/prov#>
 
         SELECT DISTINCT ?uuid ?ID
         WHERE {
@@ -337,6 +392,43 @@ def GetParticipantIDs(nidm_file_list,output_file=None):
 
     return df
 
+def GetParticipantIDFromAcquisition(nidm_file_list,acquisition, output_file=None):
+    '''
+    This function will return the participant ID of the participant with a qualified association of
+    prov:hadRole sio:Subject.
+
+    :param nidm_file_list: list of nidm files
+    :param acquisition: nidm acquisition UUID to search for qualified association
+    :param output_file: optional output filename
+    :return: a dataframe subject ID and prov:Agent UUID of participant with qualified association
+    '''
+
+    query = '''
+
+            PREFIX prov:<http://www.w3.org/ns/prov#>
+            PREFIX sio: <http://semanticscience.org/ontology/sio.owl#>
+            PREFIX ndar: <https://ndar.nih.gov/api/datadictionary/v2/dataelement/>
+            PREFIX rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>
+            PREFIX prov:<http://www.w3.org/ns/prov#>
+
+            SELECT DISTINCT ?uuid ?ID
+            WHERE {
+
+                <%s> rdf:type prov:Activity ;
+    		        prov:qualifiedAssociation _:blanknode .
+
+    	        _:blanknode prov:hadRole %s ;
+                     prov:agent ?uuid  .
+
+    	        ?uuid %s ?ID .
+
+            }
+        ''' % (acquisition, Constants.NIDM_PARTICIPANT, Constants.NIDM_SUBJECTID)
+
+    df = sparql_query_nidm(nidm_file_list, query, output_file=output_file)
+
+    return df
+
 
 def GetParticipantDetails(nidm_file_list,project_id, participant_id, output_file=None):
     '''
@@ -491,8 +583,8 @@ def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter
     ### added by DBK changed to dictionary to support subject ids along with uuids
     #participants = []
     participants = {}
-    participants["uuid"] = set()
-    participants["subject id"] = set()
+    participants["uuid"] = []
+    participants["subject id"] = []
 
 
     for file in nidm_file_list:
@@ -507,26 +599,25 @@ def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter
                     for blank in rdf_graph.objects(subject=activity, predicate=Constants.PROV['qualifiedAssociation']):
                         if (blank, Constants.PROV['hadRole'], Constants.SIO['Subject']):
                             for participant in rdf_graph.objects(subject=blank, predicate=Constants.PROV['agent']):
-                                uuid = (str(participant)).split('/')[-1]  # srip off the http://whatever/whatever/
+                                uuid = (str(participant)).split('/')[-1]  # strip off the http://whatever/whatever/
                                 if (not uuid in participants) and \
                                         ( (not filter) or CheckSubjectMatchesFilter( tuple([file]) , project, participant, filter) ):
                                     ### added by DBK for subject IDs as well ###
                                     for id in rdf_graph.objects(subject=participant,predicate=URIRef(Constants.NIDM_SUBJECTID.uri)):
-                                        subid = (str(id)).split('/')[-1]  # srip off the http://whatever/whatever/
+                                        subid = (str(id)).split('/')[-1]  # strip off the http://whatever/whatever/
 
-                                    ### added by DBK for subject IDs as well ###
-                                    #participants.append(uuid)
-                                    try:
-                                        participants['uuid'].add(uuid)
-                                        participants['subject id'].add(subid)
-                                    # just in case there's no subject id in the file...
-                                    except:
+                                        ### added by DBK for subject IDs as well ###
                                         #participants.append(uuid)
-                                        participants['uuid'].add(uuid)
-                                        participants['subject id'].add('')
+                                        if ( not uuid in participants['uuid'] ):
+                                            try:
+                                                participants['uuid'].append(uuid)
+                                                participants['subject id'].append(subid)
+                                            # just in case there's no subject id in the file...
+                                            except:
+                                                #participants.append(uuid)
+                                                participants['uuid'].append(uuid)
+                                                participants['subject id'].append('')
 
-    participants['uuid'] = list(participants['uuid'])
-    participants['subject id'] = list(participants['subject id'])
     return participants
 
 
@@ -579,12 +670,12 @@ def GetDatatypeSynonyms(nidm_file_list, project_id, datatype):
     if datatype.startswith("derivatives."):
         datatype = datatype[12:]
     project_data_elements = GetProjectDataElements(nidm_file_list, project_id)
+    all_synonyms = set([datatype])
     for dti in project_data_elements['data_type_info']:
-        if str(datatype) in [ str(x) for x in [dti['source_variable'], dti['label'], dti['datumType'], dti['measureOf'], URITail(dti['measureOf']), dti['isAbout'], URITail(dti['isAbout']), dti['dataElement'], dti['dataElementURI'], dti['prefix']] ]:
-            all_synonyms = set([str(dti['source_variable']), str(dti['label']), str(dti['datumType']), str(dti['measureOf']), URITail(dti['measureOf']), str(dti['isAbout']), str(dti['dataElement']), str(dti['dataElementURI'])] )
+        if str(datatype) in [ str(x) for x in [dti['source_variable'], dti['label'], dti['datumType'], dti['measureOf'], URITail(dti['measureOf']), str(dti['isAbout']), URITail(dti['isAbout']), dti['dataElement'], dti['dataElementURI'], dti['prefix']] ]:
+            all_synonyms = all_synonyms.union(set([str(dti['source_variable']), str(dti['label']), str(dti['datumType']), str(dti['measureOf']), URITail(dti['measureOf']), str(dti['isAbout']), str(dti['dataElement']), str(dti['dataElementURI'])] ))
             all_synonyms.remove("")  # remove the empty string in case that is in there
-            return list(all_synonyms)
-    return [datatype]
+    return all_synonyms
 
 def GetProjectDataElements(nidm_file_list, project_id):
     ### added by DBK...changing to dictionary to support labels along with uuids
@@ -706,10 +797,17 @@ def CheckSubjectMatchesFilter(nidm_file_list, project_uuid, subject_uuid, filter
             if value[0] == quote and value[-1] == quote:
                 value = value[1:-1]
 
-
         sub_pieces = splitSubject(compound_sub)
-        if len(sub_pieces) == 2 and sub_pieces[0] == 'instruments':
-            term = sub_pieces[1] # 'AGE_AT_SCAN' for example
+
+        # figure out what we are filtering on
+        term = None
+        if len(sub_pieces) == 1:
+            # no instruments or derivatives prefix was entered, so test in both
+            term = sub_pieces[0]
+
+        if (len(sub_pieces) == 2 and sub_pieces[0] == 'instruments') or len(sub_pieces) == 1:
+            if len(sub_pieces) == 2:
+                term = sub_pieces[1] # 'AGE_AT_SCAN' for example
             synonyms = GetDatatypeSynonyms(tuple(nidm_file_list), project_uuid, term)
             instrument_details = GetParticipantInstrumentData(nidm_file_list, project_uuid, subject_uuid)
             for instrument_uuid in instrument_details:
@@ -719,21 +817,19 @@ def CheckSubjectMatchesFilter(nidm_file_list, project_uuid, subject_uuid, filter
                     if found_match:
                         break
 
-        elif len(sub_pieces) == 2 and sub_pieces[0] == 'derivatives':
-            type = sub_pieces[1] # 'ilx:0102597' for example
+        if (len(sub_pieces) == 2 and sub_pieces[0] == 'derivatives') or len(sub_pieces) == 1:
+            if len(sub_pieces) == 2:
+                term = sub_pieces[1] # 'ilx:0102597' for example
             derivatives_details = GetDerivativesDataForSubject(nidm_file_list, project_uuid, subject_uuid)
             for key in derivatives_details:
                 derivatives = derivatives_details[key]['values']
                 for vkey in derivatives:  # values will be in the form { http://example.com/a/b/c#fs_00001 : { datumType: '', label: '', value: '', units:'' }, ... }
                     short_key = URITail(vkey)
-                    if short_key == type:
+                    if short_key == term:
                         found_match = filterCompare(derivatives[vkey]['value'], op, value)
                     if found_match:
                         break
 
-
-
-
         # check after each test if we got false because the tests are joined with 'and'
         if not found_match:
             return False
@@ -1078,7 +1174,7 @@ def getStatsCollectionForNode (rdf_graph, derivatives_node):
         else:
             dti = getDataTypeInfo(rdf_graph, datatype )
             if dti:  # if we can't find a datatype then this is non-data info so don't record it
-                data['values'][str(datatype)] = {'datumType': str(dti['datumType']), 'label': str(dti['label']), 'value': str(value), 'units': str(dti['hasUnit'])}
+                data['values'][str(datatype)] = {'datumType': str(dti['datumType']), 'label': str(dti['label']), 'value': str(value), 'units': str(dti['hasUnit']), 'isAbout': str(dti['isAbout'])}
 
     return data
 
@@ -1093,10 +1189,23 @@ def OpenGraph(file):
     :param file: filename
     :return: Graph
     '''
+
     # if someone passed me a RDF graph rather than a file, just send it back
     if isinstance(file, rdflib.graph.Graph):
         return file
 
+
+    # If we have a Blazegraph instance, load the data then do the rest
+    if 'BLAZEGRAPH_URL' in environ.keys():
+        try:
+            f = open(file)
+            data = f.read()
+            logging.debug("Sending {} to blazegraph".format(file))
+            r = requests.post(url=environ['BLAZEGRAPH_URL'], data=data, headers={'Content-type': 'application/x-turtle'})
+        except Exception as e:
+            logging.error("Exception {} loading {} into Blazegraph.".format(e, file))
+
+
     BLOCKSIZE = 65536
     hasher = hashlib.md5()
     with open(file, 'rb') as afile:
